@@ -9,9 +9,11 @@ import warnings
 
 try:
     str = unicode  # Python 2.* compatible
+    PY3 = False
     string_types = (basestring,)
     integer_types = (int, long)
 except NameError:
+    PY3 = True
     string_types = (str,)
     integer_types = (int,)
 
@@ -28,11 +30,9 @@ class SqlDialects(object):
     __slots__ = ('_registry', )
 
     def __init__(self):
-        """Constructor, initial registry."""
         self._registry = {}
 
     def register(self, dialect, cls):
-        """Registers callbacks."""
         def decorator(func):
             ns = self._registry.setdefault(dialect, {})
             ns[cls] = func
@@ -78,6 +78,46 @@ class Comparable(object):
 
     __slots__ = ()
 
+    def _c(op, inv=False):
+        return (lambda self, other: Condition(op, self, other)) if not inv else (lambda self, other: Condition(op, other, self))
+
+    def _ca(op, inv=False):
+        return (lambda self, *a: Constant(op)(self, *a)) if not inv else (lambda self, other: Constant(op)(other, self))
+
+    def _p(op):
+        return lambda self: Prefix(op, self)
+
+    __add__ = _c("+")
+    __radd__ = _c("+", 1)
+    __sub__ = _c("-")
+    __rsub__ = _c("-", 1)
+    __mul__ = _c("*")
+    __rmul__ = _c("*", 1)
+    __div__ = _c("/")
+    __rdiv__ = _c("/", 1)
+    __and__ = _c("AND")
+    __rand__ = _c("AND", 1)
+    __or__ = _c("OR")
+    __ror__ = _c("OR", 1)
+    __gt__ = _c(">")
+    __lt__ = _c("<")
+    __ge__ = _c(">=")
+    __le__ = _c("<=")
+    like = _c("LIKE")
+    ilike = _c("ILIKE")
+
+    __pos__ = _p("+")
+    __neg__ = _p("-")
+    __invert__ = _p("NOT")
+    distinct = _p("DISTINCT")
+
+    __pow__ = _ca("POW")
+    __rpow__ = _ca("POW", 1)
+    __mod__ = _ca("MOD")
+    __rmod__ = _ca("MOD", 1)
+    __abs__ = _ca("ABS")
+    count = _ca("COUNT")
+
     def __eq__(self, other):
         if other is None:
             return Condition("IS", self, Constant("NULL"))
@@ -92,78 +132,6 @@ class Comparable(object):
             return self.not_in(other)
         return Condition("<>", self, other)
 
-    def __add__(self, other):
-        return Condition("+", self, other)
-
-    def __radd__(self, other):
-        return Condition("+", other, self)
-
-    def __sub__(self, other):
-        return Condition("-", self, other)
-
-    def __rsub__(self, other):
-        return Condition("-", other, self)
-
-    def __mul__(self, other):
-        return Condition("*", self, other)
-
-    def __rmul__(self, other):
-        return Condition("*", other, self)
-
-    def __div__(self, other):
-        return Condition("/", self, other)
-
-    def __rdiv__(self, other):
-        return Condition("/", other, self)
-
-    def __pos__(self):
-        return Prefix("+", self)
-
-    def __neg__(self):
-        return Prefix("-", self)
-
-    def __pow__(self, other):
-        return Constant("POW")(self, other)
-
-    def __rpow__(self, other):
-        return Constant("POW")(other, self)
-
-    def __abs__(self):
-        return Constant("ABS")(self)
-
-    def __mod__(self, other):
-        return Constant("MOD")(self, other)
-
-    def __rmod__(self, other):
-        return Constant("MOD")(other, self)
-
-    def __and__(self, other):
-        return Condition("AND", self, other)
-
-    def __or__(self, other):
-        return Condition("OR", self, other)
-
-    def __rand__(self, other):
-        return Condition("AND", other, self)
-
-    def __ror__(self, other):
-        return Condition("OR", other, self)
-
-    def __invert__(self):
-        return Prefix("NOT", self)
-
-    def __gt__(self, other):
-        return Condition(">", self, other)
-
-    def __lt__(self, other):
-        return Condition("<", self, other)
-
-    def __ge__(self, other):
-        return Condition(">=", self, other)
-
-    def __le__(self, other):
-        return Condition("<=", self, other)
-
     def as_(self, alias):
         return Alias(alias, self)
 
@@ -199,12 +167,6 @@ class Comparable(object):
     def iendswith(self, other):
         return self.ilike(Concat('%', other))
 
-    def like(self, other):
-        return Condition("LIKE", self, other)
-
-    def ilike(self, other):
-        return Condition("ILIKE", self, other)
-
     def between(self, start, end):
         return Between(self, start, end)
 
@@ -214,12 +176,6 @@ class Comparable(object):
     def concat_ws(self, sep, *args):
         return Concat(self, *args).ws(sep)
 
-    def distinct(self):
-        return Prefix("DISTINCT", self)
-
-    def count(self):
-        return Constant("COUNT")(self)
-
     def asc(self):
         return Suffix(self, "ASC")
 
@@ -237,11 +193,11 @@ class Comparable(object):
 
     __hash__ = None
 
-    # Aliases:
     AS = same('as_')
     IN = same('in_')
     NOT_IN = same('not_in')
     LIKE = same('like')
+    ILIKE = same('ilike')
     BETWEEN = same('between')
 
 
@@ -263,15 +219,6 @@ class Expr(Comparable):
     def __params__(self):
         return getattr(self, '_params', [])
 
-    def __bytes__(self):
-        return sqlrepr(self).encode('utf-8')
-
-    def __str__(self):
-        return sqlrepr(self)
-
-    def __repr__(self):
-        return sqlrepr(self)
-
 
 class Condition(Expr):
 
@@ -279,12 +226,8 @@ class Condition(Expr):
 
     def __init__(self, op, expr1, expr2):
         self._op = op.upper()
-        if expr1 is not None:
-            expr1 = prepare_expr(expr1)
-        if expr2 is not None:
-            expr2 = prepare_expr(expr2)
-        self._expr1 = expr1
-        self._expr2 = expr2
+        self._expr1 = None if expr1 is None else prepare_expr(expr1)
+        self._expr2 = None if expr2 is None else prepare_expr(expr2)
 
     def __sqlrepr__(self, dialect):
         s1 = sqlrepr(self._expr1, dialect)
@@ -334,8 +277,7 @@ class ExprList(Expr):
             return self._args[key]
 
     def __iter__(self):
-        for a in self._args:
-            yield a
+        return iter(self._args)
 
     def append(self, x):
         return self._args.append(prepare_expr(x))
@@ -575,6 +517,15 @@ class Index(Alias):
 
 
 class MetaTable(type):
+
+    def __new__(cls, name, bases, attrs):
+        def _f(attr):
+            return lambda self, *a, **kw: getattr(TableJoin(self), attr)(*a, **kw)
+
+        for a in ['inner_join', 'left_join', 'right_join', 'full_join', 'cross_join', 'join', 'on', 'use_index', 'ignore_index', 'force_index']:
+            attrs[a] = _f(a)
+        return type.__new__(cls, name, bases, attrs)
+
     def __getattr__(cls, key):
         if key[0] == '_':
             raise AttributeError
@@ -598,42 +549,9 @@ class Table(MetaTable(bytes("NewBase"), (object, ), {})):
     def __init__(self, name):
         self._name = name
 
-    def inner_join(self, obj):
-        return TableJoin(self).__and__(obj)
-
-    def left_join(self, obj):
-        return TableJoin(self).__add__(obj)
-
-    def right_join(self, obj):
-        return TableJoin(self).__sub__(obj)
-
-    def full_join(self, obj):
-        return TableJoin(self).__or__(obj)
-
-    def cross_join(self, obj):
-        return TableJoin(self).__mul__(obj)
-
-    def join(self, join_type, obj):
-        return TableJoin(self).join(join_type, obj)
-
     def as_(self, alias):
         return TableAlias(alias, self)
 
-    def on(self, c):
-        return TableJoin(self).on(c)
-
-    @opt_checker(["reset", ])
-    def use_index(self, *args, **opts):
-        return TableJoin(self).use_index(*args, **opts)
-
-    @opt_checker(["reset", ])
-    def ignore_index(self, *args, **opts):
-        return TableJoin(self).ignore_index(*args, **opts)
-
-    @opt_checker(["reset", ])
-    def force_index(self, *args, **opts):
-        return TableJoin(self).force_index(*args, **opts)
-
     def __getattr__(self, name):
         if name[0] == '_':
             raise AttributeError
@@ -651,16 +569,6 @@ class Table(MetaTable(bytes("NewBase"), (object, ), {})):
     def __params__(self):
         return []
 
-    def __bytes__(self):
-        return sqlrepr(self).encode('utf-8')
-
-    def __str__(self):
-        return sqlrepr(self)
-
-    def __repr__(self):
-        return sqlrepr(self)
-
-    # Aliases:
     __and__ = same('inner_join')
     __add__ = same('left_join')
     __sub__ = same('right_join')
@@ -710,20 +618,14 @@ class TableJoin(object):
         self._ignore_index = ExprList().join(", ")
         self._force_index = ExprList().join(", ")
 
-    def inner_join(self, obj):
-        return self.join("INNER JOIN", obj)
+    def _j(j):
+        return lambda self, obj: self.join(j, obj)
 
-    def left_join(self, obj):
-        return self.join("LEFT OUTER JOIN", obj)
-
-    def right_join(self, obj):
-        return self.join("RIGHT OUTER JOIN", obj)
-
-    def full_join(self, obj):
-        return self.join("FULL OUTER JOIN", obj)
-
-    def cross_join(self, obj):
-        return self.join("CROSS JOIN", obj)
+    inner_join = _j("INNER JOIN")
+    left_join = _j("LEFT OUTER JOIN")
+    right_join = _j("RIGHT OUTER JOIN")
+    full_join = _j("FULL OUTER JOIN")
+    cross_join = _j("CROSS JOIN")
 
     def join(self, join_type, obj):
         if not isinstance(obj, TableJoin) or obj.left():
@@ -766,15 +668,12 @@ class TableJoin(object):
                 return self
         return self
 
-    @opt_checker(["reset", ])
     def use_index(self, *args, **opts):
         return self.change_index(self._use_index, *args, **opts)
 
-    @opt_checker(["reset", ])
     def ignore_index(self, *args, **opts):
         return self.change_index(self._ignore_index, *args, **opts)
 
-    @opt_checker(["reset", ])
     def force_index(self, *args, **opts):
         return self.change_index(self._force_index, *args, **opts)
 
@@ -810,16 +709,6 @@ class TableJoin(object):
     def __params__(self):
         return sqlparams(self._left) + sqlparams(self._table) + sqlparams(self._on)
 
-    def __bytes__(self):
-        return sqlrepr(self).encode('utf-8')
-
-    def __str__(self):
-        return sqlrepr(self)
-
-    def __repr__(self):
-        return sqlrepr(self)
-
-    # Aliases:
     as_nested = same('group')
     __and__ = same('inner_join')
     __add__ = same('left_join')
@@ -1218,7 +1107,6 @@ class QuerySet(Expr):
         sql = self._build_sql()
         return sqlparams(sql)
 
-    # Aliases:
     columns = same('fields')
     __copy__ = same('clone')
 
@@ -1292,9 +1180,7 @@ def parentheses_conditional(expr):
 def prepare_expr(expr):
     if expr is None:
         return Constant("NULL")
-    expr = placeholder_conditional(expr)
-    expr = parentheses_conditional(expr)
-    return expr
+    return parentheses_conditional(placeholder_conditional(expr))
 
 
 def default_dialect(dialect=None):
@@ -1333,13 +1219,11 @@ const = ConstantSpace()
 func = const
 qn = Name()
 
-# Python 2.* compatible
-try:
-    unicode
-except NameError:
-    pass
-else:
-    for cls in (Expr, Table, TableJoin, ):
+for cls in (Expr, Table, TableJoin, ):
+    cls.__bytes__ = lambda self: sqlrepr(self).encode('utf-8')
+    cls.__str__ = lambda self: sqlrepr(self)
+    cls.__repr__ = lambda self: "<{0}: {1}, {2}>".format(type(self).__name__, sqlrepr(self), sqlparams(self))
+    if not PY3:
         cls.__unicode__ = cls.__str__
         cls.__str__ = cls.__bytes__
 
